@@ -1,22 +1,18 @@
-#!/usr/bin/python3
+"""Tools for creating a CA cert and signed server certs.
 
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-
-Tools for creating a CA cert and signed server certs.
 Divined from http://svn.osafoundation.org/m2crypto/trunk/tests/test_x509.py
 The mk_temporary_xxx calls return a NamedTemporaryFile with certs.
-Usage ;
-   # Create a temporary CA cert and it's private key
-   cacert, cakey = mk_temporary_cacert()
-   # Create a temporary server cert+key, signed by the CA
-   server_cert = mk_temporary_cert(cacert.name, cakey.name, '*.server.co.uk')
+Usage:
+  # Create a temporary CA cert and it's private key
+  cacert, cakey = mk_temporary_cacert()
+  # Create a temporary server cert+key, signed by the CA
+  server_cert = mk_temporary_cert(cacert.name, cakey.name, '*.server.co.uk')
+
+Protips:
+  # openssl verify -CAfile cacert.crt cacert.crt cert.crt
+  # openssl x509 -in cert.crt -noout -text
+  # openssl x509 -in cacert.crt -noout -text
 
-protips
-# openssl verify -CAfile cacert.crt cacert.crt cert.crt
-# openssl x509 -in cert.crt -noout -text
-# openssl x509 -in cacert.crt -noout -text
 """
 
 import argparse
@@ -33,6 +29,7 @@ import time
 import zipfile
 from http.server import BaseHTTPRequestHandler, HTTPServer
 from socketserver import ThreadingMixIn
+from typing import Any, Dict, List, Optional, Tuple, Union
 
 import yaml
 
@@ -40,23 +37,31 @@ try:
     from yaml import CSafeDumper as SafeDumper
     from yaml import CSafeLoader as SafeLoader
 except ImportError:
-    from yaml import SafeLoader, SafeDumper
+    from yaml import SafeLoader, SafeDumper  # type: ignore
 
 from cryptography import exceptions as crypto_exceptions
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import serialization
-from cryptography.hazmat.primitives.asymmetric import padding
+from cryptography.hazmat.primitives.asymmetric import padding, rsa
+from cryptography.hazmat.primitives.asymmetric.dsa import DSAPrivateKey
+from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKey
+from cryptography.hazmat.primitives.asymmetric.ed448 import Ed448PrivateKey
+from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey
+from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
+from cryptography.x509 import Certificate
+from cryptography.x509.extensions import ExtensionNotFound
+from cryptography.x509.general_name import UniformResourceIdentifier
 
 from keylime import ca_impl_openssl as ca_impl
 from keylime import cmd_exec, config, crypto, fs_util, json, keylime_logging, revocation_notifier
 
 logger = keylime_logging.init_logging("ca-util")
 
-global_password = None
+global_password: Optional[str] = None
 
 
-def load_cert_by_path(cert_path):
+def load_cert_by_path(cert_path: str) -> Certificate:
     cert = None
     with open(cert_path, "rb") as ca_file:
         cert = x509.load_pem_x509_certificate(
@@ -66,30 +71,43 @@ def load_cert_by_path(cert_path):
     return cert
 
 
-def setpassword(pw):
+def setpassword(pw: Optional[str]) -> None:
     global global_password
-    if len(pw) == 0:
+    if not pw:
+        pw = getpass.getpass("Please enter the password to decrypt your keystore: ")
+
+    if not pw:
         raise Exception("You must specify a password!")
+
     global_password = pw
 
 
-def cmd_mkcert(workingdir, name):
+def cmd_mkcert(workingdir: str, name: str, password: Optional[str] = None) -> None:
     cwd = os.getcwd()
+    mask = os.umask(0o037)
     try:
         fs_util.ch_dir(workingdir)
         priv = read_private()
         cacert = load_cert_by_path("cacert.crt")
         ca_pk = serialization.load_pem_private_key(priv[0]["ca"], password=None, backend=default_backend())
+        if not isinstance(
+            ca_pk, (EllipticCurvePrivateKey, RSAPrivateKey, DSAPrivateKey, Ed448PrivateKey, Ed25519PrivateKey)
+        ):
+            raise Exception(
+                f"Private key of type {type(ca_pk).__name__} cannot be used for creating an x509 certificate"
+            )
 
         cert, pk = ca_impl.mk_signed_cert(cacert, ca_pk, name, priv[0]["lastserial"] + 1)
 
-        with open(f"{name}-cert.crt", "wb") as f:
+        with os.fdopen(os.open(f"{name}-cert.crt", os.O_WRONLY | os.O_CREAT, 0o640), "wb") as f:
             f.write(cert.public_bytes(serialization.Encoding.PEM))
 
         priv[0][name] = pk.private_bytes(
             encoding=serialization.Encoding.PEM,
             format=serialization.PrivateFormat.PKCS8,
-            encryption_algorithm=serialization.NoEncryption(),
+            encryption_algorithm=serialization.BestAvailableEncryption(password.encode("utf-8"))
+            if password
+            else serialization.NoEncryption(),
         )
 
         # increment serial number after successful creation
@@ -97,10 +115,10 @@ def cmd_mkcert(workingdir, name):
 
         write_private(priv)
 
-        with os.fdopen(os.open(f"{name}-private.pem", os.O_WRONLY | os.O_CREAT, 0o600), "wb") as f:
+        with os.fdopen(os.open(f"{name}-private.pem", os.O_WRONLY | os.O_CREAT, 0o640), "wb") as f:
             f.write(priv[0][name])
 
-        with os.fdopen(os.open(f"{name}-public.pem", os.O_WRONLY | os.O_CREAT, 0o600), "wb") as f:
+        with os.fdopen(os.open(f"{name}-public.pem", os.O_WRONLY | os.O_CREAT, 0o640), "wb") as f:
             f.write(
                 pk.public_key().public_bytes(
                     encoding=serialization.Encoding.PEM, format=serialization.PublicFormat.SubjectPublicKeyInfo
@@ -109,6 +127,8 @@ def cmd_mkcert(workingdir, name):
 
         cc = load_cert_by_path(f"{name}-cert.crt")
         pubkey = cacert.public_key()
+        assert isinstance(pubkey, rsa.RSAPublicKey)
+        assert cc.signature_hash_algorithm is not None
         pubkey.verify(
             cc.signature,
             cc.tbs_certificate_bytes,
@@ -120,11 +140,13 @@ def cmd_mkcert(workingdir, name):
     except crypto_exceptions.InvalidSignature:
         logger.error("ERROR: Cert does not validate against CA")
     finally:
+        os.umask(mask)
         os.chdir(cwd)
 
 
-def cmd_init(workingdir):
+def cmd_init(workingdir: str) -> None:
     cwd = os.getcwd()
+    mask = os.umask(0o037)
     try:
         fs_util.ch_dir(workingdir)
 
@@ -153,7 +175,7 @@ def cmd_init(workingdir):
 
         write_private(priv)
 
-        with os.fdopen(os.open("ca-public.pem", os.O_WRONLY | os.O_CREAT, 0o600), "wb") as f:
+        with os.fdopen(os.open("ca-public.pem", os.O_WRONLY | os.O_CREAT, 0o640), "wb") as f:
             f.write(
                 ca_pk.public_key().public_bytes(
                     encoding=serialization.Encoding.PEM, format=serialization.PublicFormat.SubjectPublicKeyInfo
@@ -174,6 +196,8 @@ def cmd_init(workingdir):
         # Sanity checks...
         cac = load_cert_by_path("cacert.crt")
         pubkey = cacert.public_key()
+        assert isinstance(pubkey, rsa.RSAPublicKey)
+        assert cac.signature_hash_algorithm is not None
         pubkey.verify(
             cac.signature,
             cac.tbs_certificate_bytes,
@@ -186,9 +210,10 @@ def cmd_init(workingdir):
         logger.error("ERROR: Cert does not self validate")
     finally:
         os.chdir(cwd)
+        os.umask(mask)
 
 
-def cmd_certpkg(workingdir, name, insecure=False):
+def cmd_certpkg(workingdir: str, name: str, insecure: bool = False) -> Tuple[bytes, int, str]:
     cwd = os.getcwd()
     try:
         fs_util.ch_dir(workingdir)
@@ -255,22 +280,22 @@ def cmd_certpkg(workingdir, name, insecure=False):
         os.chdir(cwd)
 
 
-def convert_crl_to_pem(derfile, pemfile):
+def convert_crl_to_pem(derfile: str, pemfile: str) -> None:
     with open(derfile, "rb") as der_f, open(pemfile, "wb") as pem_f:
         der_crl = der_f.read()
         pem_f.write(x509.load_der_x509_crl(der_crl).public_bytes(encoding=serialization.Encoding.PEM))
 
 
-def get_crl_distpoint(cert_path):
+def get_crl_distpoint(cert_path: str) -> Optional[str]:
     cert_obj = load_cert_by_path(cert_path)
 
     try:
         crl_distpoints = cert_obj.extensions.get_extension_for_class(x509.CRLDistributionPoints).value
         for dstpnt in crl_distpoints:
             for point in dstpnt.full_name:
-                if isinstance(point, x509.general_name.UniformResourceIdentifier):
+                if isinstance(point, UniformResourceIdentifier):
                     return point.value
-    except x509.extensions.ExtensionNotFound:
+    except ExtensionNotFound:
         pass
 
     logger.info("No CRL distribution points in %s", cert_path)
@@ -280,7 +305,7 @@ def get_crl_distpoint(cert_path):
 # to check: openssl crl -inform DER -text -noout -in cacrl.der
 
 
-def cmd_revoke(workingdir, name=None, serial=None):
+def cmd_revoke(workingdir: str, name: Optional[str] = None, serial: Optional[int] = None) -> bytes:
     cwd = os.getcwd()
     try:
         fs_util.ch_dir(workingdir)
@@ -296,15 +321,15 @@ def cmd_revoke(workingdir, name=None, serial=None):
             serial = cert.serial_number
 
         # convert serial to string
-        serial = str(serial)
+        serial_str = str(serial)
 
         # get the ca key cert and keys as strings
         with open("cacert.crt", encoding="utf-8") as f:
             cacert = f.read()
         ca_pk = priv[0]["ca"].decode("utf-8")
 
-        if serial not in priv[0]["revoked_keys"]:
-            priv[0]["revoked_keys"].append(serial)
+        if serial_str not in priv[0]["revoked_keys"]:
+            priv[0]["revoked_keys"].append(serial_str)
 
         crl = ca_impl.gencrl(priv[0]["revoked_keys"], cacert, ca_pk)
 
@@ -324,7 +349,7 @@ def cmd_revoke(workingdir, name=None, serial=None):
 # regenerate the crl without revoking anything
 
 
-def cmd_regencrl(workingdir):
+def cmd_regencrl(workingdir: str) -> bytes:
     cwd = os.getcwd()
     try:
         fs_util.ch_dir(workingdir)
@@ -349,7 +374,7 @@ def cmd_regencrl(workingdir):
     return crl
 
 
-def cmd_listen(workingdir, cert_path):
+def cmd_listen(workingdir: str, cert_path: str) -> None:
     cwd = os.getcwd()
     try:
         fs_util.ch_dir(workingdir)
@@ -366,7 +391,7 @@ def cmd_listen(workingdir, cert_path):
         logger.info("Hosting CRL on %s:%d", socket.getfqdn(), config.CRL_PORT)
         t.start()
 
-        def check_expiration():
+        def check_expiration() -> None:
             logger.info("checking CRL for expiration every hour")
             while True:  # pylint: disable=R1702
                 try:
@@ -393,7 +418,7 @@ def cmd_listen(workingdir, cert_path):
         t2 = threading.Thread(target=check_expiration, daemon=True)
         t2.start()
 
-        def revoke_callback(revocation):
+        def revoke_callback(revocation: Dict[str, Union[str, bytes]]) -> None:
             json_meta = json.loads(revocation["meta_data"])
             serial = json_meta["cert_serial"]
             if revocation.get("type", None) != "revocation" or serial is None:
@@ -422,14 +447,15 @@ def cmd_listen(workingdir, cert_path):
 class ThreadedCRLServer(ThreadingMixIn, HTTPServer):
     published_crl = None
 
-    def setcrl(self, crl):
+    def setcrl(self, crl: bytes) -> None:
         self.published_crl = crl
 
 
 class CRLHandler(BaseHTTPRequestHandler):
-    def do_GET(self):
+    def do_GET(self) -> None:
         logger.info("GET invoked from %s with uri: %s", str(self.client_address), self.path)
 
+        assert isinstance(self.server, ThreadedCRLServer)
         if self.server.published_crl is None:
             self.send_response(404)
             self.end_headers()
@@ -440,32 +466,36 @@ class CRLHandler(BaseHTTPRequestHandler):
             self.wfile.write(self.server.published_crl)
 
 
-def rmfiles(path):
+def rmfiles(path: str) -> None:
     files = glob.glob(path)
     for f in files:
         os.remove(f)
 
 
-def write_private(inp):
+def write_private(inp: Tuple[Dict[str, Any], str]) -> None:
     priv = inp[0]
     salt = inp[1]
 
     priv_encoded = yaml.dump(priv, Dumper=SafeDumper)
+    assert isinstance(global_password, str)
     key = crypto.kdf(global_password, salt)
-    ciphertext = crypto.encrypt(priv_encoded, key)
+    ciphertext = crypto.encrypt(priv_encoded.encode("utf-8"), key)
     towrite = {"salt": salt, "priv": ciphertext}
 
-    with os.fdopen(os.open("private.yml", os.O_WRONLY | os.O_CREAT, 0o600), "w", encoding="utf-8") as f:
+    mask = os.umask(0o037)
+    with os.fdopen(os.open("private.yml", os.O_WRONLY | os.O_CREAT, 0o640), "w", encoding="utf-8") as f:
         yaml.dump(towrite, f, Dumper=SafeDumper)
+    os.umask(mask)
 
 
-def read_private(warn=False):
+def read_private(warn: bool = False) -> Tuple[Dict[str, Any], str]:
     if global_password is None:
         setpassword(getpass.getpass("Please enter the password to decrypt your keystore: "))
 
     if os.path.exists("private.yml"):
         with open("private.yml", encoding="utf-8") as f:
             toread = yaml.load(f, Loader=SafeLoader)
+        assert isinstance(global_password, str)
         key = crypto.kdf(global_password, toread["salt"])
         try:
             plain = crypto.decrypt(toread["priv"], key)
@@ -478,10 +508,10 @@ def read_private(warn=False):
         # file doesn't exist, just invent a salt
         logger.warning("Private certificate data %s does not exist yet.", os.path.abspath("private.yml"))
         logger.warning("Keylime will attempt to load private certificate data again when it is needed.")
-    return {"revoked_keys": []}, base64.b64encode(crypto.generate_random_key()).decode()
+    return {"revoked_keys": [], "ca": b""}, base64.b64encode(crypto.generate_random_key()).decode()
 
 
-def main(argv=sys.argv):  # pylint: disable=dangerous-default-value
+def main(argv: List[str] = sys.argv) -> None:  # pylint: disable=dangerous-default-value
     parser = argparse.ArgumentParser(argv[0])
     parser.add_argument(
         "-c",
