@@ -1,16 +1,9 @@
-#!/usr/bin/python3
-
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 from keylime import keylime_agent, keylime_logging
 
 logger = keylime_logging.init_logging("cloudagent")
 
 
-def main():
+def main() -> None:
     keylime_agent.main()
 
 
