@@ -1,49 +1,54 @@
-#!/usr/bin/python3
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 import argparse
 import codecs
 import itertools
 import select
 import sys
 import time
+from typing import Optional
 
+from keylime import config
 from keylime.common import algorithms
 from keylime.ima import ast
-from keylime.tpm.tpm_abstract import config
 from keylime.tpm.tpm_main import tpm
 
 # Instaniate tpm
 tpm_instance = tpm(need_hw_tpm=True)
 
 
-def measure_list(file_path, position, ima_hash_alg, pcr_hash_alg, search_val=None):
+def measure_list(
+    file_path: str,
+    position: int,
+    ima_hash_alg: algorithms.Hash,
+    pcr_hash_alg: algorithms.Hash,
+    search_val: Optional[str] = None,
+) -> int:
     with open(file_path, encoding="utf-8") as f:
         lines = itertools.islice(f, position, None)
 
         runninghash = ast.get_START_HASH(pcr_hash_alg)
 
+        search_val_bytes = None
         if search_val is not None:
-            search_val = codecs.decode(search_val.encode("utf-8"), "hex")
+            search_val_bytes = codecs.decode(search_val.encode("utf-8"), "hex")
 
         for line in lines:
-            line = line.strip()
+            # remove only the newline character, as there can be the
+            # space as the delimiter character followed by an empty
+            # field at the end
+            line = line.strip("\n")
             position += 1
 
             entry = ast.Entry(line, None, ima_hash_alg=ima_hash_alg, pcr_hash_alg=pcr_hash_alg)
 
-            if search_val is None:
+            if search_val_bytes is None:
                 val = codecs.encode(entry.pcr_template_hash, "hex").decode("utf8")
                 tpm_instance.extendPCR(config.IMA_PCR, val, pcr_hash_alg)
             else:
                 runninghash = pcr_hash_alg.hash(runninghash + entry.pcr_template_hash)
-                if runninghash == search_val:
+                if runninghash == search_val_bytes:
                     return position
 
-        if search_val is not None:
+        if search_val_bytes is not None:
             raise Exception(
                 "Unable to find current measurement list position, Resetting the TPM emulator may be neccesary"
             )
@@ -51,12 +56,12 @@ def measure_list(file_path, position, ima_hash_alg, pcr_hash_alg, search_val=Non
     return position
 
 
-def main(argv=sys.argv):  # pylint: disable=dangerous-default-value
+def main() -> None:
     parser = argparse.ArgumentParser()
     parser.add_argument("-a", "--hash_algs", nargs="*", default=["sha1"], help="PCR banks hash algorithms")
     parser.add_argument("-i", "--ima-hash-alg", default="sha1", help="Set hash algorithm that is used in IMA log")
     parser.add_argument("-f", "--ima-log", default=config.IMA_ML, help="path to the IMA log")
-    args = parser.parse_args(argv[1:])
+    args = parser.parse_args()
 
     if not tpm_instance.is_emulator():
         raise Exception("This stub should only be used with a TPM emulator")
@@ -68,7 +73,12 @@ def main(argv=sys.argv):  # pylint: disable=dangerous-default-value
         position[pcr_hash_alg] = 0
 
     for pcr_hash_alg in dict(position):
-        pcr_val = tpm_instance.readPCR(config.IMA_PCR, pcr_hash_alg)
+        try:
+            pcr_val = tpm_instance.readPCR(config.IMA_PCR, pcr_hash_alg)
+        except Exception as ex:
+            print(f"Error: {ex}")
+            sys.exit(1)
+
         if codecs.decode(pcr_val.encode("utf-8"), "hex") != ast.get_START_HASH(pcr_hash_alg):
             print(
                 f"Warning: IMA PCR is not empty for hash algorithm {pcr_hash_alg}, "
@@ -99,4 +109,4 @@ def main(argv=sys.argv):  # pylint: disable=dangerous-default-value
 
 
 if __name__ == "__main__":
-    main(sys.argv)
+    main()
