@@ -1,10 +1,6 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2020 Kaifeng Wang
-"""
 import enum
 import hashlib
-from typing import Any, List, Optional
+from typing import Any, List, cast
 
 
 def is_accepted(algorithm: str, accepted: List[Any]) -> bool:
@@ -17,40 +13,35 @@ def is_accepted(algorithm: str, accepted: List[Any]) -> bool:
 
 
 class Hash(str, enum.Enum):
+    # Names compatible with tpm2-tools (man/common/alg.md)
     SHA1 = "sha1"
     SHA256 = "sha256"
     SHA384 = "sha384"
     SHA512 = "sha512"
     SM3_256 = "sm3_256"
 
+    @classmethod
+    def from_algorithm(cls, algorithm: str) -> "Hash":
+        return cls(algorithm)
+
     @staticmethod
     def is_recognized(algorithm: str) -> bool:
         try:
-            Hash(algorithm)
-            return True
+            Hash(algorithm).get_size()
         except ValueError:
             return False
+        return True
 
-    def hash(self, data: bytes) -> Optional[bytes]:
-        if self == Hash.SHA1:
-            return hashlib.sha1(data).digest()
-        if self == Hash.SHA256:
-            return hashlib.sha256(data).digest()
-        if self == Hash.SHA384:
-            return hashlib.sha384(data).digest()
-        if self == Hash.SHA512:
-            return hashlib.sha512(data).digest()
-        if self == Hash.SM3_256:
-            # SM3 might not be guaranteed to be there
-            try:
-                return hashlib.new("sm3", data).digest()
-            except ValueError:
-                return None
+    def __hashfn(self, data: bytes) -> Any:
+        # Translate from tmp2-tools name into hashlib.new() name
+        alg = "sm3" if self.value == "sm3_256" else self.value
+        return hashlib.new(alg, data)
 
-        return None
+    def hash(self, data: bytes) -> bytes:
+        return cast(bytes, self.__hashfn(data).digest())
 
-    def get_size(self) -> Optional[int]:
-        return _HASH_SIZE.get(self)
+    def get_size(self) -> int:
+        return cast(int, self.__hashfn(b"").digest_size * 8)
 
     def __str__(self) -> str:
         return self.value
@@ -77,12 +68,3 @@ class Sign:
     @staticmethod
     def is_recognized(algorithm: str) -> bool:
         return algorithm in Sign.supported_algorithms
-
-
-_HASH_SIZE = {
-    Hash.SHA1: 160,
-    Hash.SHA256: 256,
-    Hash.SHA384: 384,
-    Hash.SHA512: 512,
-    Hash.SM3_256: 256,
-}
