@@ -1,8 +1,3 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2020 Luke Hinds (lhinds@redhat.com), Red Hat, Inc.
-"""
-
 from sqlalchemy import Column, Integer, PickleType, String, Text
 from sqlalchemy.ext.declarative import declarative_base
 
@@ -11,7 +6,7 @@ from keylime.json import JSONPickler
 Base = declarative_base()
 
 
-class JSONPickleType(PickleType):  # pylint: disable=abstract-method
+class JSONPickleType(PickleType):  # pylint: disable=abstract-method,too-many-ancestors
     impl = Text
     cache_ok = True
 
