@@ -1,14 +1,10 @@
-#!/usr/bin/env python3
-"""
-SPDX-License-Identifier: Apache-2.0
-"""
-
 import base64
 import enum
 import json
 import struct
+from typing import Any, Dict, List, Optional, Tuple, Union
 
-from cryptography import utils, x509
+from cryptography import x509
 from cryptography.exceptions import InvalidSignature, UnsupportedAlgorithm
 from cryptography.hazmat import backends
 from cryptography.hazmat.primitives import hashes, serialization
@@ -16,12 +12,16 @@ from cryptography.hazmat.primitives.asymmetric import ec, padding
 from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePublicKey
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
 from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
+from cryptography.x509 import Certificate, oid
+from cryptography.x509.extensions import ExtensionNotFound, SubjectKeyIdentifier
 
 from keylime import keylime_logging
 
 logger = keylime_logging.init_logging("file_signatures")
 
 
+SupportedKeyTypes = Union[RSAPublicKey, EllipticCurvePublicKey]
+
 """
 Tools for IMA file signature verification
 """
@@ -55,22 +55,20 @@ class HashAlgo(enum.IntEnum):
 
 
 # Streebog is supported by evmctl
-@utils.register_interface(hashes.HashAlgorithm)
-class MyStreebog256:
+class MyStreebog256(hashes.HashAlgorithm):
     """Basic class for Streebog256"""
 
-    name = "streebog256"
-    digest_size = 32
-    block_size = 64
+    name = "streebog256"  # type: ignore
+    digest_size = 32  # type: ignore
+    block_size = 64  # type: ignore
 
 
-@utils.register_interface(hashes.HashAlgorithm)
-class MyStreebog512:
+class MyStreebog512(hashes.HashAlgorithm):
     """Basic class for Streebog512"""
 
-    name = "streebog512"
-    digest_size = 64
-    block_size = 64
+    name = "streebog512"  # type: ignore
+    digest_size = 64  # type: ignore
+    block_size = 64  # type: ignore
 
 
 HASH_FUNCS = {
@@ -105,12 +103,14 @@ class PubkeyAlgo(enum.IntEnum):
 class ImaKeyring:
     """ImaKeyring models an IMA keyring where keys are indexed by their keyid"""
 
-    def __init__(self):
+    ringv2: Dict[int, SupportedKeyTypes]
+
+    def __init__(self) -> None:
         """Constructor"""
         self.ringv2 = {}
 
     @staticmethod
-    def _get_keyidv2(pubkey):
+    def _get_keyidv2(pubkey: SupportedKeyTypes) -> int:
         """Calculate the keyidv2 of a given public key object. The keyidv2
         are the lowest 4 bytes of the sha1 hash over the public key bytes
         of a DER-encoded key in PKCS1 format.
@@ -130,7 +130,7 @@ class ImaKeyring:
         keydigest = digest.finalize()
         return int.from_bytes(keydigest[16:], "big")
 
-    def add_pubkey(self, pubkey, keyidv2):
+    def add_pubkey(self, pubkey: SupportedKeyTypes, keyidv2: Optional[int]) -> None:
         """Add a public key object to the keyring; a keyidv2 may be passed in
         and if it is 'None' it will be determined using the commonly used
         sha1 hash function for calculating the Subject Key Identifier.
@@ -141,33 +141,33 @@ class ImaKeyring:
         self.ringv2[keyidv2] = pubkey
         logger.debug("Added key with keyid: 0x%08x", keyidv2)
 
-    def get_pubkey_by_keyidv2(self, keyidv2):
+    def get_pubkey_by_keyidv2(self, keyidv2: int) -> Optional[SupportedKeyTypes]:
         """Get a public key object given its keyidv2"""
         return self.ringv2.get(keyidv2)
 
-    def to_json(self):
+    def to_json(self) -> Dict[str, Union[List[int], List[str]]]:
         """Convert the ImaKeyring into a JSON object"""
         fmt = serialization.PublicFormat.SubjectPublicKeyInfo
-        obj = {}
+        obj: Dict[str, Union[List[int], List[str]]] = {}
         lst = []
 
         for pubkey in self.ringv2.values():
             try:
                 pubbytes = pubkey.public_bytes(encoding=serialization.Encoding.DER, format=fmt)
+                lst.append(pubbytes)
             except Exception as ex:
                 logger.error("Could not serialize key: %s", str(ex))
-            lst.append(pubbytes)
 
         obj["pubkeys"] = [base64.b64encode(pubkey).decode("ascii") for pubkey in lst]
         obj["keyids"] = list(self.ringv2.keys())
         return obj
 
-    def to_string(self):
+    def to_string(self) -> str:
         """Generate a string representation"""
         return json.dumps(self.to_json())
 
     @staticmethod
-    def _base64_to_der_keylist(base64_keylist, keyidv2_list):
+    def _base64_to_der_keylist(base64_keylist: List[str], keyidv2_list: List[int]) -> List[Tuple[bytes, Optional[int]]]:
         """Convert a base64-encoded list of public keys to a list of DER-encoded
         public keys; a keyidv2_list may also be given that contains
         the keyidv2 of each key
@@ -179,7 +179,7 @@ class ImaKeyring:
         return res
 
     @staticmethod
-    def from_string(stringrepr):
+    def from_string(stringrepr: str) -> Optional["ImaKeyring"]:
         """Convert a string-encoded ImaKeyring to an ImaKeyring object"""
         if not stringrepr:
             return None
@@ -188,8 +188,8 @@ class ImaKeyring:
 
         default_be = backends.default_backend()
 
-        # An empty Db entry comes as a string '[]'. A valid DB entry as a string
-        # ith escaped quotes and needs to be loaded twice
+        # An empty Db entry comes as a string 'null' or previously '[]'. A valid DB entry
+        # is a string with escaped quotes and needs to be loaded twice
         obj = json.loads(stringrepr)
         if isinstance(obj, str):
             obj = json.loads(obj)
@@ -198,9 +198,11 @@ class ImaKeyring:
 
         keyids = obj.get("keyids", [])
 
-        for (der_key, keyidv2) in ImaKeyring._base64_to_der_keylist(obj["pubkeys"], keyids):
+        for der_key, keyidv2 in ImaKeyring._base64_to_der_keylist(obj["pubkeys"], keyids):
             try:
                 pubkey = serialization.load_der_public_key(der_key, backend=default_be)
+                if not isinstance(pubkey, (RSAPublicKey, EllipticCurvePublicKey)):
+                    raise ValueError(f"Unsupported key type {type(pubkey).__name__}")
                 ima_keyring.add_pubkey(pubkey, keyidv2)
             except Exception as ex:
                 logger.error("Could not load a base64-decoded DER key: %s", str(ex))
@@ -210,18 +212,23 @@ class ImaKeyring:
 class ImaKeyrings:
     """IMA Keyrings models the various keyrings of the system where IMA may take its keys from"""
 
-    def __init__(self):
+    keyrings: Dict[str, ImaKeyring]
+
+    def __init__(self) -> None:
         """Constructor"""
         self.keyrings = {}
 
-    def add_to_keyring_from_data(self, filedata, keyring_name):
+    def add_to_keyring_from_data(self, filedata: bytes, keyring_name: str) -> None:
         """Add the public key, given as a plain filedata (bytes), to the keyring by the given name
         after converting the key to an object. If a keyring by the given name doesn't exist, one
         will be created."""
         pubkey, keyidv2 = get_pubkey(filedata)
-        self.add_pubkey_to_keyring(pubkey, keyring_name, keyidv2=keyidv2)
+        if pubkey:
+            self.add_pubkey_to_keyring(pubkey, keyring_name, keyidv2=keyidv2)
 
-    def add_pubkey_to_keyring(self, pubkey, keyring_name, keyidv2=None):
+    def add_pubkey_to_keyring(
+        self, pubkey: SupportedKeyTypes, keyring_name: str, keyidv2: Optional[int] = None
+    ) -> None:
         """Add a public key object to a keyring by the given name. If a keyring by the given name
         doesn't exist, one will be created."""
         keyring = self.keyrings.get(keyring_name)
@@ -230,22 +237,22 @@ class ImaKeyrings:
             self.keyrings[keyring_name] = keyring
         keyring.add_pubkey(pubkey, keyidv2)
 
-    def set_tenant_keyring(self, tenant_keyring):
+    def set_tenant_keyring(self, tenant_keyring: Optional[ImaKeyring]) -> None:
         """Set the tenant keyring for which the tenant provided the keys via command line."""
         if tenant_keyring:
             self.keyrings["tenant_keyring"] = tenant_keyring
         else:
             self.keyrings.pop("tenant_keyring", None)
 
-    def get_tenant_keyring(self):
+    def get_tenant_keyring(self) -> Optional[ImaKeyring]:
         """Get the tenant keyring."""
         return self.keyrings.get("tenant_keyring")
 
-    def get_all_keyrings(self):
+    def get_all_keyrings(self) -> List[ImaKeyring]:
         """Get a list of all the keyrings"""
         return list(self.keyrings.values())
 
-    def to_json(self):
+    def to_json(self) -> Dict[str, str]:
         """Convert an ImaKeyrings into its JSON representation; this does not include the tenant keyring"""
         obj = {}
 
@@ -256,12 +263,12 @@ class ImaKeyrings:
 
         return obj
 
-    def to_string(self):
+    def to_string(self) -> str:
         """Convert an ImaKeyrings into its string representation; this does not include the tenant keyring"""
         return json.dumps(self.to_json())
 
     @staticmethod
-    def from_string(stringrepr):
+    def from_string(stringrepr: str) -> Optional["ImaKeyrings"]:
         """Convert a string-encoded ImaKeyrings into an ImaKeyrings object."""
 
         obj = json.loads(stringrepr)
@@ -273,24 +280,26 @@ class ImaKeyrings:
         return ImaKeyrings.from_json(obj)
 
     @staticmethod
-    def from_json(obj):
+    def from_json(obj: Dict[str, str]) -> "ImaKeyrings":
         """Convert a JSON representation of an ImaKeyrings object to an ImaKeyrings object."""
         ima_keyrings = ImaKeyrings()
 
         for name, keyring_str in obj.items():
-            ima_keyrings.keyrings[name] = ImaKeyring.from_string(keyring_str)
+            ima_keyring = ImaKeyring.from_string(keyring_str)
+            if ima_keyring:
+                ima_keyrings.keyrings[name] = ima_keyring
 
         return ima_keyrings
 
     @staticmethod
-    def _verify(pubkey, sig, filehash, hashfunc):
+    def _verify(pubkey: SupportedKeyTypes, sig: bytes, filehash: bytes, hashfunc: hashes.HashAlgorithm) -> None:
         """Do signature verification with the given public key"""
         if isinstance(pubkey, RSAPublicKey):
             pubkey.verify(sig, filehash, padding.PKCS1v15(), Prehashed(hashfunc))
         elif isinstance(pubkey, EllipticCurvePublicKey):
             pubkey.verify(sig, filehash, ec.ECDSA(Prehashed(hashfunc)))
 
-    def _asymmetric_verify(self, signature, filehash, filehash_type):
+    def _asymmetric_verify(self, signature: bytes, filehash: bytes, filehash_type: str) -> bool:
         """Do an IMA signature verification given the signature data from
         the log, which is formatted as 'struct signature_v2_hdr'.
         This function resembles the kernel code:
@@ -342,7 +351,7 @@ class ImaKeyrings:
             return False
         return True
 
-    def integrity_digsig_verify(self, signature, filehash, filehash_type):
+    def integrity_digsig_verify(self, signature: bytes, filehash: bytes, filehash_type: str) -> bool:
         """Validate the signature against the given hash trying all keyrings.
         This function resembles the kernel code at:
         https://elixir.bootlin.com/linux/v5.9/source/security/integrity/digsig.c#L59
@@ -364,7 +373,7 @@ class ImaKeyrings:
         return False
 
 
-def _get_pubkey_from_der_public_key(filedata, backend):
+def _get_pubkey_from_der_public_key(filedata: bytes, backend: Any) -> Tuple[Any, None]:
     """Load the filedata as a DER public key"""
     try:
         return serialization.load_der_public_key(filedata, backend=backend), None
@@ -372,7 +381,7 @@ def _get_pubkey_from_der_public_key(filedata, backend):
         return None, None
 
 
-def _get_pubkey_from_pem_public_key(filedata, backend):
+def _get_pubkey_from_pem_public_key(filedata: bytes, backend: Any) -> Tuple[Any, None]:
     """Load the filedata as a PEM public key"""
     try:
         return serialization.load_pem_public_key(filedata, backend=backend), None
@@ -380,7 +389,7 @@ def _get_pubkey_from_pem_public_key(filedata, backend):
         return None, None
 
 
-def _get_pubkey_from_der_private_key(filedata, backend):
+def _get_pubkey_from_der_private_key(filedata: bytes, backend: Any) -> Tuple[Any, None]:
     """Load the filedata as a DER private key"""
     try:
         privkey = serialization.load_der_private_key(filedata, None, backend=backend)
@@ -389,7 +398,7 @@ def _get_pubkey_from_der_private_key(filedata, backend):
         return None, None
 
 
-def _get_pubkey_from_pem_private_key(filedata, backend):
+def _get_pubkey_from_pem_private_key(filedata: bytes, backend: Any) -> Tuple[Any, None]:
     """Load the filedata as a PEM private key"""
     try:
         privkey = serialization.load_pem_private_key(filedata, None, backend=backend)
@@ -398,20 +407,23 @@ def _get_pubkey_from_pem_private_key(filedata, backend):
         return None, None
 
 
-def _get_keyidv2_from_cert(cert):
-    """Get the keyidv2 from the cert's Subject Key Identifier"""
-    if not cert.extensions:
-        return None
+def _get_keyidv2_from_cert(cert: Certificate) -> Optional[int]:
+    """Get the keyidv2 from the cert's Subject Key Identifier (SKID) if available."""
+    if cert.extensions:
+        try:
+            skid = cert.extensions.get_extension_for_oid(oid.ExtensionOID.SUBJECT_KEY_IDENTIFIER)
+            if skid and skid.value and isinstance(skid.value, SubjectKeyIdentifier) and len(skid.value.digest) >= 4:
+                keyidv2 = int.from_bytes(skid.value.digest[-4:], "big")
+                logger.debug("Extracted keyidv2 from cert: 0x%08x", keyidv2)
+                return keyidv2
+        except ExtensionNotFound:
+            pass
 
-    skid = cert.extensions.get_extension_for_oid(x509.oid.ExtensionOID.SUBJECT_KEY_IDENTIFIER)
-    if skid and skid.value and len(skid.value.digest) >= 4:
-        keyidv2 = int.from_bytes(skid.value.digest[-4:], "big")
-        logger.debug("Extracted keyidv2 from cert: 0x%08x", keyidv2)
-        return keyidv2
+    logger.warning("Certificate doesn't have a Subject Key Identifier; keyidv2 will be derived from public key")
     return None
 
 
-def _get_pubkey_from_der_x509_certificate(filedata, backend):
+def _get_pubkey_from_der_x509_certificate(filedata: bytes, backend: Any) -> Tuple[Any, Optional[int]]:
     """Load the filedata as a DER x509 certificate"""
     try:
         cert = x509.load_der_x509_certificate(filedata, backend=backend)
@@ -420,7 +432,7 @@ def _get_pubkey_from_der_x509_certificate(filedata, backend):
         return None, None
 
 
-def _get_pubkey_from_pem_x509_certificate(filedata, backend):
+def _get_pubkey_from_pem_x509_certificate(filedata: bytes, backend: Any) -> Tuple[Any, Optional[int]]:
     """Load the filedata as a PEM x509 certificate"""
     try:
         cert = x509.load_pem_x509_certificate(filedata, backend=backend)
@@ -429,7 +441,7 @@ def _get_pubkey_from_pem_x509_certificate(filedata, backend):
         return None, None
 
 
-def get_pubkey(filedata):
+def get_pubkey(filedata: bytes) -> Tuple[Optional[SupportedKeyTypes], Optional[int]]:
     """Get the public key from the filedata; if an x509 certificate is
     given, also determine the keyidv2 from the Subject Key Identifier,
     otherwise return None
@@ -449,20 +461,21 @@ def get_pubkey(filedata):
     ]:
         pubkey, keyidv2 = func(filedata, default_be)
         if pubkey:
+            if not isinstance(pubkey, (RSAPublicKey, EllipticCurvePublicKey)):
+                raise ValueError(f"Unsupported key type {type(pubkey).__name__}")
             return pubkey, keyidv2
 
     return None, None
 
 
-def get_pubkey_from_file(filename):
+def get_pubkey_from_file(filename: str) -> Tuple[Optional[SupportedKeyTypes], Optional[int]]:
     """Get the public key object from a file"""
-    try:
-        with open(filename, "rb") as fobj:
-            filedata = fobj.read()
-            pubkey, keyidv2 = get_pubkey(filedata)
-            if pubkey:
-                return pubkey, keyidv2
-    except Exception:
-        pass
+    with open(filename, "rb") as fobj:
+        filedata = fobj.read()
+        pubkey, keyidv2 = get_pubkey(filedata)
+        if pubkey:
+            if not isinstance(pubkey, (RSAPublicKey, EllipticCurvePublicKey)):
+                raise ValueError(f"Unsupported key type {type(pubkey).__name__}")
+            return pubkey, keyidv2
 
     return None, None
