@@ -1,29 +1,54 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 import codecs
 import copy
-import datetime
 import functools
 import hashlib
 import json
-import os
-import re
-from typing import Optional
+from typing import Any, Dict, List, Optional, Pattern, Set, TextIO, Tuple, Type
 
 from keylime import config, keylime_logging, signing
 from keylime.agentstates import AgentAttestState
 from keylime.common import algorithms, validators
+from keylime.common.algorithms import Hash
 from keylime.failure import Component, Failure
 from keylime.ima import ast, file_signatures, ima_dm
+from keylime.ima.file_signatures import ImaKeyrings
 
 logger = keylime_logging.init_logging("ima")
 
 
-# The version of the allowlist format that is supported by this keylime release
-ALLOWLIST_CURRENT_VERSION = 5
+# The version of the IMA policy format that is supported by this keylime release
+RUNTIME_POLICY_CURRENT_VERSION = 1
+
+
+class RUNTIME_POLICY_GENERATOR:
+    Unknown = 0
+    EmptyAllowList = 1
+    CompatibleAllowList = 2
+    LegacyAllowList = 3
+
+
+# A correctly formatted empty IMA policy.
+#
+# Older versions of Keylime allowlists did not have the "log_hash_alg"
+# parameter. Hard-coding it to "sha1" is perfectly fine, and the fact one
+# specifies a different algorithm on the kernel command line (e.g., ima_hash=sha256)
+# does not affect normal operation of Keylime, since it does not validate the
+# hash algorithm received from agent's IMA runtime measurements.
+# The only situation where this hard-coding would become a problem is if and when
+# the kernel maintainers decide to use a different algorithm for template-hash.
+EMPTY_RUNTIME_POLICY = {
+    "meta": {
+        "version": RUNTIME_POLICY_CURRENT_VERSION,
+        "generator": RUNTIME_POLICY_GENERATOR.EmptyAllowList,
+    },
+    "release": 0,
+    "digests": {},
+    "excludes": [],
+    "keyrings": {},
+    "ima": {"ignored_keyrings": [], "log_hash_alg": "sha1", "dm_policy": None},
+    "ima-buf": {},
+    "verification-keys": "",
+}
 
 
 class IMAMeasurementList:
@@ -32,24 +57,25 @@ class IMAMeasurementList:
     """
 
     instance = None
+    entries: Set[Tuple[int, int]]
 
     @staticmethod
-    def get_instance():
+    def get_instance() -> "IMAMeasurementList":
         """Return a singleton"""
         if not IMAMeasurementList.instance:
             IMAMeasurementList.instance = IMAMeasurementList()
         return IMAMeasurementList.instance
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Constructor"""
         self.entries = set()
         self.reset()
 
-    def reset(self):
+    def reset(self) -> None:
         """Reset the variables"""
         self.entries = set()
 
-    def update(self, num_entries, filesize):
+    def update(self, num_entries: int, filesize: int) -> None:
         """Update the number of entries and current filesize of the log."""
         if len(self.entries) > 256:
             for entry in self.entries:
@@ -57,7 +83,7 @@ class IMAMeasurementList:
                 break
         self.entries.add((num_entries, filesize))
 
-    def find(self, nth_entry):
+    def find(self, nth_entry: int) -> Tuple[int, int]:
         """Find the closest entry to the n-th entry and return its number
         and filesize to seek to, return 0, 0 if nothing was found.
         """
@@ -68,7 +94,7 @@ class IMAMeasurementList:
         return best
 
 
-def read_measurement_list(ima_log_file, nth_entry):
+def read_measurement_list(ima_log_file: TextIO, nth_entry: int) -> Tuple[Optional[str], int, int]:
     """Read the IMA measurement list starting from a given entry.
     The entry may be of any value 0 <= entry <= entries_in_log where
     entries_in_log + 1 indicates that the client wants to read the next entry
@@ -113,7 +139,13 @@ def read_measurement_list(ima_log_file, nth_entry):
     return ml, nth_entry, num_entries
 
 
-def _validate_ima_ng(exclude_regex, allowlist, digest: ast.Digest, path: ast.Name, hash_types="hashes") -> Failure:
+def _validate_ima_ng(
+    exclude_regex: Optional[Pattern[str]],
+    allowlist: Optional[Dict[str, Any]],
+    digest: ast.Digest,
+    path: ast.Name,
+    hash_types: str = "digests",
+) -> Failure:
     failure = Failure(Component.IMA, ["validation", "ima-ng"])
     if allowlist is not None:
         if exclude_regex is not None and exclude_regex.match(path.name):
@@ -148,12 +180,16 @@ def _validate_ima_ng(exclude_regex, allowlist, digest: ast.Digest, path: ast.Nam
 
 
 def _validate_ima_sig(
-    exclude_regex, ima_keyrings, allowlist, digest: ast.Digest, path: ast.Name, signature: ast.Signature
+    exclude_regex: Optional[Pattern[str]],
+    ima_keyrings: Optional[file_signatures.ImaKeyrings],
+    allowlist: Optional[Dict[str, Any]],
+    digest: ast.Digest,
+    path: ast.Name,
+    signature: ast.Signature,
 ) -> Failure:
     failure = Failure(Component.IMA, ["validator", "ima-sig"])
     valid_signature = False
     if ima_keyrings and signature:
-
         if exclude_regex is not None and exclude_regex.match(path.name):
             logger.debug("IMA: ignoring excluded path %s", path.name)
             return failure
@@ -171,8 +207,8 @@ def _validate_ima_sig(
     # - the signature is valid and the file is also in the allowlist
     if (
         allowlist is not None
-        and allowlist.get("hashes") is not None
-        and ((allowlist["hashes"].get(path.name, None) is not None and valid_signature) or not valid_signature)
+        and allowlist.get("digests") is not None
+        and ((allowlist["digests"].get(path.name, None) is not None and valid_signature) or not valid_signature)
     ):
         # We use the normal ima_ng validator to validate hash
         return _validate_ima_ng(exclude_regex, allowlist, digest, path)
@@ -187,24 +223,33 @@ def _validate_ima_sig(
 
 
 def _validate_ima_buf(
-    exclude_regex,
-    allowlist,
-    ima_keyrings: file_signatures.ImaKeyrings,
+    exclude_regex: Optional[Pattern[str]],
+    allowlist: Optional[Dict[str, Any]],
+    ima_keyrings: Optional[file_signatures.ImaKeyrings],
     dm_validator: Optional[ima_dm.DmIMAValidator],
     digest: ast.Digest,
     path: ast.Name,
     data: ast.Buffer,
-):
+) -> Failure:
     failure = Failure(Component.IMA)
     # Is data.data a key?
-    pubkey, keyidv2 = file_signatures.get_pubkey(data.data)
+    try:
+        pubkey, keyidv2 = file_signatures.get_pubkey(data.data)
+    except ValueError as ve:
+        failure.add_event("invalid_key", f"key from {path.name} does not have a supported key: {ve}", True)
+        return failure
+
     if pubkey:
-        ignored_keyrings = allowlist["ima"]["ignored_keyrings"]
+        ignored_keyrings = []
+        if allowlist:
+            ignored_keyrings = allowlist.get("ima", {}).get("ignored_keyrings", [])
+
         if "*" not in ignored_keyrings and path.name not in ignored_keyrings:
             failure = _validate_ima_ng(exclude_regex, allowlist, digest, path, hash_types="keyrings")
             if not failure:
                 # Add the key only now that it's validated (no failure)
-                ima_keyrings.add_pubkey_to_keyring(pubkey, path.name, keyidv2=keyidv2)
+                if ima_keyrings is not None:
+                    ima_keyrings.add_pubkey_to_keyring(pubkey, path.name, keyidv2=keyidv2)
     # Check if this is a device mapper entry only if we have a validator for that
     elif dm_validator is not None and path.name in dm_validator.valid_names:
         failure = dm_validator.validate(digest, path, data)
@@ -217,45 +262,49 @@ def _validate_ima_buf(
 
 
 def _process_measurement_list(
-    agentAttestState, lines, hash_alg, lists=None, m2w=None, pcrval=None, ima_keyrings=None, boot_aggregates=None
-):
+    agentAttestState: AgentAttestState,
+    lines: List[str],
+    hash_alg: Hash,
+    runtime_policy: Optional[Dict[str, Any]] = None,
+    pcrval: Optional[str] = None,
+    ima_keyrings: Optional[ImaKeyrings] = None,
+    boot_aggregates: Optional[Dict[str, List[str]]] = None,
+) -> Tuple[str, Failure]:
     failure = Failure(Component.IMA)
     running_hash = agentAttestState.get_pcr_state(config.IMA_PCR, hash_alg)
+    assert running_hash
+
     found_pcr = pcrval is None
-    errors = {}
+    errors: Dict[Type[ast.Mode], int] = {}
     pcrval_bytes = b""
     if pcrval is not None:
         pcrval_bytes = codecs.decode(pcrval.encode("utf-8"), "hex")
 
-    if lists is not None:
-        if isinstance(lists, str):
-            lists = json.loads(lists)
-        allow_list = lists["allowlist"]
-        exclude_list = lists["exclude"]
+    if runtime_policy is not None:
+        exclude_list = runtime_policy.get("excludes")
     else:
-        allow_list = None
         exclude_list = None
 
     ima_log_hash_alg = algorithms.Hash.SHA1
-    if allow_list is not None:
+    if runtime_policy is not None:
         try:
-            ima_log_hash_alg = algorithms.Hash(allow_list["ima"]["log_hash_alg"])
+            ima_log_hash_alg = algorithms.Hash(runtime_policy["ima"]["log_hash_alg"])
         except ValueError:
             logger.warning(
                 "Specified IMA log hash algorithm %s is not a valid algorithm! Defaulting to SHA1.",
-                allow_list["ima"]["log_hash_alg"],
+                runtime_policy["ima"]["log_hash_alg"],
             )
 
-    if boot_aggregates and allow_list:
-        if "boot_aggregate" not in allow_list["hashes"]:
-            allow_list["hashes"]["boot_aggregate"] = []
+    if boot_aggregates and runtime_policy:
+        if "boot_aggregate" not in runtime_policy["digests"]:
+            runtime_policy["digests"]["boot_aggregate"] = []
         for alg in boot_aggregates.keys():
             for val in boot_aggregates[alg]:
-                if val not in allow_list["hashes"]["boot_aggregate"]:
-                    allow_list["hashes"]["boot_aggregate"].append(val)
+                if val not in runtime_policy["digests"]["boot_aggregate"]:
+                    runtime_policy["digests"]["boot_aggregate"].append(val)
 
-    is_valid, compiled_regex, err_msg = validators.valid_exclude_list(exclude_list)
-    if not is_valid:
+    exclude_list_compiled_regex, err_msg = validators.valid_exclude_list(exclude_list)
+    if err_msg:
         # This should not happen as the exclude list has already been validated
         # by the verifier before acceping it. This is a safety net just in case.
         err_msg += " Exclude list will be ignored."
@@ -263,8 +312,8 @@ def _process_measurement_list(
 
     # Setup device mapper validation
     dm_validator = None
-    if allow_list is not None:
-        dm_policy = allow_list["ima"]["dm_policy"]
+    if runtime_policy is not None:
+        dm_policy = runtime_policy["ima"]["dm_policy"]
 
         if dm_policy is not None:
             dm_validator = ima_dm.DmIMAValidator(dm_policy)
@@ -275,22 +324,29 @@ def _process_measurement_list(
 
     ima_validator = ast.Validator(
         {
-            ast.ImaSig: functools.partial(_validate_ima_sig, compiled_regex, ima_keyrings, allow_list),
-            ast.ImaNg: functools.partial(_validate_ima_ng, compiled_regex, allow_list),
-            ast.Ima: functools.partial(_validate_ima_ng, compiled_regex, allow_list),
-            ast.ImaBuf: functools.partial(_validate_ima_buf, compiled_regex, allow_list, ima_keyrings, dm_validator),
+            ast.ImaSig: functools.partial(_validate_ima_sig, exclude_list_compiled_regex, ima_keyrings, runtime_policy),
+            ast.ImaNg: functools.partial(_validate_ima_ng, exclude_list_compiled_regex, runtime_policy),
+            ast.Ima: functools.partial(_validate_ima_ng, exclude_list_compiled_regex, runtime_policy),
+            ast.ImaBuf: functools.partial(
+                _validate_ima_buf, exclude_list_compiled_regex, runtime_policy, ima_keyrings, dm_validator
+            ),
         }
     )
 
     # Iterative attestation may send us no log [len(lines) == 1]; compare last know PCR 10 state
     # against current PCR state.
-    # Since IMA log append and PCR extend is not atomic, we may get a quote that does not yet take
-    # into account the next appended measurement's [len(lines) == 2] PCR extension.
-    if not found_pcr and len(lines) <= 2:
+    # Since IMA's append to the log and PCR extend as well as Keylime's retrieval of the quote, reading
+    # of PCR 10 and retrieval of the log are not atomic, we may get a quote that does not yet take into
+    # account the next-appended measurements' [len(lines) >= 2] PCR extension(s). In fact, the value of
+    # the PCR may lag the log by several entries.
+    if not found_pcr:
         found_pcr = running_hash == pcrval_bytes
 
     for linenum, line in enumerate(lines):
-        line = line.strip()
+        # remove only the newline character, as there can be the space
+        # as the delimiter character followed by an empty field at the
+        # end
+        line = line.strip("\n")
         if line == "":
             continue
 
@@ -317,11 +373,6 @@ def _process_measurement_list(
                     if dm_validator:
                         agentAttestState.set_ima_dm_state(dm_validator.state_dump())
 
-            # Keep old functionality for writing the parsed files with hashes into a file
-            if m2w is not None and (type(entry.mode) in [ast.Ima, ast.ImaNg, ast.ImaSig]):
-                hash_value = codecs.encode(entry.mode.digest.bytes, "hex")
-                path = entry.mode.path.name
-                m2w.write(f"{hash_value} {path}\n")
         except ast.ParserError:
             failure.add_event("entry", f"Line was not parsable into a valid IMA entry: {line}", True, ["parser"])
             logger.error("Line was not parsable into a valid IMA entry: %s", line)
@@ -341,23 +392,21 @@ def _process_measurement_list(
 
 
 def process_measurement_list(
-    agentAttestState,
-    lines,
-    lists=None,
-    m2w=None,
-    pcrval=None,
-    ima_keyrings=None,
-    boot_aggregates=None,
-    hash_alg=algorithms.Hash.SHA1,
-):
+    agentAttestState: AgentAttestState,
+    lines: List[str],
+    runtime_policy: Optional[Dict[str, Any]] = None,
+    pcrval: Optional[str] = None,
+    ima_keyrings: Optional[ImaKeyrings] = None,
+    boot_aggregates: Optional[Dict[str, List[str]]] = None,
+    hash_alg: algorithms.Hash = algorithms.Hash.SHA1,
+) -> Tuple[str, Failure]:
     failure = Failure(Component.IMA)
     try:
         running_hash, failure = _process_measurement_list(
             agentAttestState,
             lines,
             hash_alg,
-            lists=lists,
-            m2w=m2w,
+            runtime_policy=runtime_policy,
             pcrval=pcrval,
             ima_keyrings=ima_keyrings,
             boot_aggregates=boot_aggregates,
@@ -372,220 +421,145 @@ def process_measurement_list(
     return running_hash, failure
 
 
-def update_allowlist(allowlist):
-    """Update the allowlist to the latest version adding default values for missing fields"""
-    allowlist["meta"]["version"] = ALLOWLIST_CURRENT_VERSION
+# Read IMA policy files from disk, validate signatures and checksums, and prepare for sending.
+def read_runtime_policy(
+    alist: Optional[str] = None,
+    checksum: Optional[str] = "",
+    al_sig_file: Optional[str] = None,
+    al_key_file: Optional[str] = None,
+) -> Tuple[bytes, bytes, bytes]:
+    al_key = b""
+    al_sig = b""
+    verify_signature = False
 
-    # version 2 added 'keyrings'
-    if "keyrings" not in allowlist:
-        allowlist["keyrings"] = {}
-    # version 3 added 'ima' map with 'ignored_keyrings'
-    if "ima" not in allowlist:
-        allowlist["ima"] = {}
-    if not "ignored_keyrings" in allowlist["ima"]:
-        allowlist["ima"]["ignored_keyrings"] = []
-    # version 4 added 'ima-buf'
-    if "ima-buf" not in allowlist:
-        allowlist["ima-buf"] = {}
-    # version 5 added 'log_hash_alg'
-    if not "log_hash_alg" in allowlist["ima"]:
-        allowlist["ima"]["log_hash_alg"] = "sha1"
-    # version 6 added 'dm_policy'
-    if "dm_policy" not in allowlist["ima"]:
-        allowlist["ima"]["dm_policy"] = None
+    # If user only wants signatures then a runtime policy is not required
+    if alist is None or alist == "":
+        alist_bytes = json.dumps(copy.deepcopy(EMPTY_RUNTIME_POLICY)).encode()
 
-    return allowlist
+    elif isinstance(alist, str):
+        with open(alist, "rb") as alist_f:
+            logger.debug("Loading runtime policy from %s", alist)
+            alist_bytes = alist_f.read()
 
-
-def process_allowlists(allowlist, exclude):
-    # Pull in default config values if not specified
-    if allowlist is None:
-        allowlist = read_allowlist()
     else:
-        allowlist = update_allowlist(allowlist)
+        raise Exception("Invalid runtime policy provided")
 
-    if allowlist["ima"]["dm_policy"] is not None:
-        logger.info("IMA device mapper support is still experimental and might change in the future!")
+    # Load signatures/keys if needed
+    if al_sig_file and al_key_file:
+        logger.debug(
+            "Loading key (%s) and signature (%s) checking against runtime policy (%s)", al_key_file, al_sig_file, alist
+        )
+        verify_signature = True
+        with open(al_key_file, "rb") as key_f:
+            al_key = key_f.read()
+        with open(al_sig_file, "rb") as sig_f:
+            al_sig = sig_f.read()
 
-    if exclude is None:
-        exclude = read_excllist()
+    # Verify runtime policy. This function checks for correct JSON formatting, and
+    # will also verify signatures if provided.
+    try:
+        verify_runtime_policy(alist_bytes, al_key, al_sig, verify_sig=verify_signature)
+    except ImaValidationError as error:
+        message = f"Validation for runtime policy {alist} failed! Error: {error.message}"
+        raise Exception(message) from error
 
-    if allowlist["hashes"].get("boot_aggregate") is None:
-        logger.warning("No boot_aggregate value found in allowlist, adding an empty one")
-        allowlist["hashes"]["boot_aggregate"] = ["0" * 40, "0" * 64]
-
-    for excl in exclude:
-        # remove commented out lines
-        if excl.startswith("#"):
-            exclude.remove(excl)
-        # don't allow empty lines in exclude list, it will match everything
-        if excl == "":
-            exclude.remove(excl)
-
-    return {"allowlist": allowlist, "exclude": exclude}
-
-
-# IMA allowlists of versions older than 5 will not have the "log_hash_alg"
-# parameter. Hard-coding it to "sha1" is perfectly fine, and the fact one
-# specifies a different algorithm on the kernel command line (e.g., ima_hash=sha256)
-# does not affect normal operation of Keylime, since it does not validate the
-# hash algorithm received from agent's IMA runtime measurements.
-# The only situation where this hard-coding would become a problem is if and when
-# the kernel maintainers decide to use a different algorithm for template-hash.
-empty_allowlist = {
-    "meta": {
-        "version": ALLOWLIST_CURRENT_VERSION,
-    },
-    "release": 0,
-    "hashes": {},
-    "keyrings": {},
-    "ima": {"ignored_keyrings": [], "log_hash_alg": "sha1"},
-}
-
-
-def read_allowlist(al_path=None, checksum="", gpg_sig_file=None, gpg_key_file=None):
-    if al_path is None:
-        al_path = config.get("tenant", "ima_allowlist")
-
-    # If user only wants signatures then an allowlist is not required
-    if al_path is None or al_path == "":
-        return copy.deepcopy(empty_allowlist)
-
-    # Purposefully die if path doesn't exist
-    with open(al_path, "rb") as f:
-        pass
-
-    # verify GPG signature if needed
-    if gpg_sig_file and gpg_key_file:
-        signing.verify_signature_from_file(gpg_key_file, al_path, gpg_sig_file, "allowlist")
-
-    # Purposefully die if path doesn't exist
-    with open(al_path, "rb") as f:
-        logger.debug("Loading allowlist from %s", al_path)
-        alist_bytes = f.read()
-        sha256 = hashlib.sha256()
-        sha256.update(alist_bytes)
-        calculated_checksum = sha256.hexdigest()
-        alist_raw = alist_bytes.decode("utf-8")
-        logger.debug("Loaded allowlist from %s with checksum %s", al_path, calculated_checksum)
+    sha256 = hashlib.sha256()
+    sha256.update(alist_bytes)
+    calculated_checksum = sha256.hexdigest()
+    logger.debug("Loaded runtime policy from %s with checksum %s", alist, calculated_checksum)
 
     if checksum:
         if checksum == calculated_checksum:
-            logger.debug("Allowlist passed checksum validation")
+            logger.debug("Runtime policy passed checksum validation")
         else:
             raise Exception(
-                f"Checksum of allowlist does not match! Expected {checksum}, Calculated {calculated_checksum}"
+                f"Checksum of runtime policy does not match! Expected {checksum}, Calculated {calculated_checksum}"
             )
 
-    # if the first non-whitespace character in the file is '{' treat it as the new JSON format
-    p = re.compile(r"^\s*{")
-    if p.match(alist_raw):
-        logger.debug("Reading allow list as JSON format")
-        alist = json.loads(alist_raw)
+    return alist_bytes, al_key, al_sig
 
-        # verify it's the current version
-        if "meta" in alist and "version" in alist["meta"]:
-            version = alist["meta"]["version"]
-            if int(version) <= ALLOWLIST_CURRENT_VERSION:
-                logger.debug("Allowlist has compatible version %s", version)
-            else:
-                # in the future we will support multiple versions and convert between them,
-                # but for now there is only one
-                raise Exception("Allowlist has unsupported version {version}")
-        else:
-            logger.debug("Allowlist does not specify a version. Assuming current version %s", ALLOWLIST_CURRENT_VERSION)
 
+def runtime_policy_db_contents(runtime_policy_name: str, runtime_policy: str, tpm_policy: str = "") -> Dict[str, Any]:
+    """Assembles a runtime policy dictionary to be written on the database"""
+    runtime_policy_db_format: Dict[str, Any] = {}
+    runtime_policy_db_format["name"] = runtime_policy_name
+    # TODO: This was required to ensure e2e CI tests pass
+    if runtime_policy == "{}":
+        runtime_policy_db_format["ima_policy"] = None
     else:
-        # convert legacy format into new structured format
-        logger.debug("Converting legacy allowlist format to JSON")
+        runtime_policy_db_format["ima_policy"] = runtime_policy
+    runtime_policy_bytes = runtime_policy.encode()
+    runtime_policy_dict = deserialize_runtime_policy(runtime_policy)
+    if "meta" in runtime_policy_dict:
+        if "generator" in runtime_policy_dict["meta"]:
+            runtime_policy_db_format["generator"] = runtime_policy_dict["meta"]["generator"]
+        else:
+            runtime_policy_db_format["generator"] = RUNTIME_POLICY_GENERATOR.Unknown
+    if tpm_policy:
+        runtime_policy_db_format["tpm_policy"] = tpm_policy
 
-        alist = copy.deepcopy(empty_allowlist)
-        alist["meta"]["timestamp"] = str(datetime.datetime.now())
-        alist["meta"]["generator"] = "keylime-legacy-format-upgrade"
-        if checksum:
-            alist["meta"]["checksum"] = checksum
+    runtime_policy_db_format["checksum"] = hashlib.sha256(runtime_policy_bytes).hexdigest()
 
-        for line in alist_raw.splitlines():
-            line = line.strip()
-            if len(line) == 0:
-                continue
-
-            pieces = line.split(None, 1)
-            if not len(pieces) == 2:
-                logger.warning("Line in AllowList does not consist of hash and file path: %s", line)
-                continue
-
-            (checksum_hash, path) = pieces
-
-            if path.startswith("%keyring:"):
-                entrytype = "keyrings"
-                path = path[len("%keyring:") :]  # remove leading '%keyring:' from path to get keyring name
-            else:
-                entrytype = "hashes"
-
-            if path in alist[entrytype]:
-                alist[entrytype][path].append(checksum_hash)
-            else:
-                alist[entrytype][path] = [checksum_hash]
-
-    alist = update_allowlist(alist)
-
-    return alist
+    return runtime_policy_db_format
 
 
-def read_excllist(exclude_path=None):
-    if exclude_path is None:
-        exclude_path = config.get("tenant", "ima_excludelist")
-
-    excl_list = []
-    if os.path.exists(exclude_path):
-        with open(exclude_path, encoding="utf-8") as f:
-            for line in f:
-                line = line.strip()
-                if line.startswith("#") or len(line) == 0:
-                    continue
-                excl_list.append(line)
-
-        logger.debug("Loaded exclusion list from %s: %s", exclude_path, excl_list)
-
-    return excl_list
+class ImaValidationError(Exception):
+    def __init__(self, message: str, code: int):
+        self.message = message
+        self.code = code
+        super().__init__(self.message)
 
 
-def main():
-    allowlist_path = "allowlist.txt"
-    print(f"reading allowlist from {allowlist_path}")
+def verify_runtime_policy(
+    runtime_policy: bytes,
+    runtime_policy_key: Optional[bytes] = None,
+    runtime_policy_sig: Optional[bytes] = None,
+    verify_sig: Optional[bool] = True,
+) -> None:
+    """
+    Verify that a runtime policy is valid. If provided runtime policy has a detached signature, verify the signature.
+    """
 
-    exclude_path = "exclude.txt"
-    # exclude_path = '../scripts/ima/exclude.txt'
-    print(f"reading exclude list from {exclude_path}")
+    if runtime_policy is None:
+        raise ImaValidationError(
+            message="No IMA policy provided!",
+            code=400,
+        )
 
-    al_data = read_allowlist(allowlist_path)
-    excl_data = read_excllist(exclude_path)
-    lists = process_allowlists(al_data, excl_data)
+    if verify_sig:
+        if not runtime_policy_sig:
+            raise ImaValidationError(
+                message="Verifier is enforcing signature validation, but no signature was provided.", code=405
+            )
+        if not runtime_policy_key:
+            raise ImaValidationError(
+                message="Verifier is enforcing signature validation, but no public key was provided.", code=405
+            )
 
-    measure_path = config.IMA_ML
-    # measure_path='../scripts/ima/ascii_runtime_measurements_ima'
-    # measure_path = '../scripts/gerardo/ascii_runtime_measurements'
-    print(f"reading measurement list from {measure_path}")
-    with open(measure_path, encoding="ascii") as f:
-        lines = f.readlines()
+        if not signing.verify_signature(runtime_policy_key, runtime_policy_sig, runtime_policy):
+            raise ImaValidationError(message="Runtime policy failed detached signature verification!", code=401)
+        logger.info("Runtime policy passed detached signature verification")
 
-        with open("measure2allow.txt", "w", encoding="utf-8") as m2a:
-            digest = process_measurement_list(AgentAttestState("1"), lines, lists, m2a)
-            print(f"final digest is {digest}")
-
-    print("using m2a")
-
-    al_data = read_allowlist("measure2allow.txt")
-    excl_data = read_excllist(exclude_path)
-    lists2 = process_allowlists(al_data, excl_data)
-    process_measurement_list(AgentAttestState("2"), lines, lists2)
-
-    print("done")
-
-
-if __name__ == "__main__":
+    # validate that the allowlist is proper JSON
     try:
-        main()
-    except Exception as e:
-        logger.exception(e)
+        lists = json.loads(runtime_policy)
+    except Exception as error:
+        raise ImaValidationError(message="Runtime policy is not valid JSON!", code=400) from error
+
+    # Validate exclude list contains valid regular expressions
+    _, excl_err_msg = validators.valid_exclude_list(lists.get("exclude"))
+    if excl_err_msg:
+        raise ImaValidationError(
+            message=f"{excl_err_msg} Exclude list regex is misformatted. Please correct the issue and try again.",
+            code=400,
+        )
+
+
+def deserialize_runtime_policy(runtime_policy: str) -> Dict[str, Any]:
+    """
+    Converts policies stored in the database to JSON (if applicable), for use in code.
+    """
+
+    # TODO: Extract IMA policy JSON from DSSE envelope, if applicable.
+    runtime_policy_deserialized: Dict[str, Any] = json.loads(runtime_policy)
+    return runtime_policy_deserialized
