@@ -1,8 +1,3 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2021 IBM Corporation
-"""
-
 import tempfile
 import unittest
 
@@ -10,7 +5,7 @@ from keylime.ima import ima
 
 
 class TestIMA(unittest.TestCase):
-    def test_read_measurement_list(self):
+    def test_read_measurement_list(self) -> None:
         filedata = "0-entry\n1-entry\n2-entry\n"
         with tempfile.NamedTemporaryFile(delete=True) as tf:
             tf.write(filedata.encode("utf-8"))
@@ -18,11 +13,12 @@ class TestIMA(unittest.TestCase):
 
             # Open file again to get str on read()
             with open(tf.name, "r", encoding="utf-8") as ima_log_file:
-
                 # Request the 2nd entry, which is available
                 ml, nth_entry, num_entries = ima.read_measurement_list(ima_log_file, 2)
                 self.assertEqual(num_entries, 3)
                 self.assertEqual(nth_entry, 2)
+                self.assertIsNotNone(ml)
+                assert ml is not None  # redundant but it makes type-checking happy
                 self.assertTrue(ml.startswith("2-entry"))
 
                 # Request the 3rd entry, which is not available yet, thus we get an empty list
@@ -36,6 +32,8 @@ class TestIMA(unittest.TestCase):
                 ml, nth_entry, num_entries = ima.read_measurement_list(ima_log_file, 4)
                 self.assertEqual(num_entries, 3)
                 self.assertEqual(nth_entry, 0)
+                self.assertIsNotNone(ml)
+                assert ml is not None  # redundant but it makes type-checking happy
                 self.assertTrue(ml.startswith("0-entry"))
 
 
