@@ -1,86 +1,56 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 import logging
-import os
+import ssl
 import sys
+from typing import Any, Dict, Optional, Union
 
 from keylime import api_version as keylime_api_version
-from keylime import config, crypto, json, keylime_logging
+from keylime import crypto, json, keylime_logging
 from keylime.requests_client import RequestsClient
 
+if sys.version_info >= (3, 8):
+    from typing import TypedDict
+else:
+    from typing_extensions import TypedDict
+
+if sys.version_info >= (3, 11):
+    from typing import NotRequired
+else:
+    from typing_extensions import NotRequired
+
+
+class RegistrarData(TypedDict):
+    ip: Optional[str]
+    port: Optional[str]
+    regcount: int
+    mtls_cert: Optional[str]
+    aik_tpm: str
+    ek_tpm: str
+    ekcert: Optional[str]
+    provider_keys: NotRequired[Dict[str, str]]
+
+
 logger = keylime_logging.init_logging("registrar_client")
-tls_cert_info = ()
-ca_cert = False
-tls_enabled = False
 api_version = keylime_api_version.current_version()
 
 
-def init_client_tls(section):
-    global tls_cert_info
-    global tls_enabled
-    global ca_cert
+def getData(
+    registrar_ip: str, registrar_port: str, agent_id: str, tls_context: Optional[ssl.SSLContext]
+) -> Optional[RegistrarData]:
+    """
+    Get the agent data from the registrar.
 
-    # make this reentrant
-    if tls_cert_info:
-        return
+    This is called by the tenant code
 
-    if not config.getboolean("general", "enable_tls"):
-        logger.warning("Warning: TLS is currently disabled, AIKs may not be authentic.")
-        return
-
-    logger.warning("TLS is enabled.")
-    tls_enabled = True
-
-    logger.info("Setting up client TLS...")
-    tls_dir = config.get(section, "registrar_tls_dir")
-
-    ca_cert = config.get(section, "registrar_ca_cert")
-    my_cert = config.get(section, "registrar_my_cert")
-    my_priv_key = config.get(section, "registrar_private_key")
-
-    if tls_dir == "default":
-        tls_dir = "reg_ca"
-        ca_cert = "cacert.crt"
-        my_cert = "client-cert.crt"
-        my_priv_key = "client-private.pem"
-
-    if tls_dir == "CV":
-        tls_dir = "cv_ca"
-        ca_cert = "cacert.crt"
-        my_cert = "client-cert.crt"
-        my_priv_key = "client-private.pem"
-
-    # this is relative path, convert to absolute in WORK_DIR
-    if tls_dir[0] != "/":
-        tls_dir = os.path.abspath(os.path.join(config.WORK_DIR, tls_dir))
-
-    if not os.path.isabs(ca_cert):
-        ca_cert = os.path.join(tls_dir, ca_cert)
-
-    if os.path.isabs(my_cert):
-        tls_cert = my_cert
-    else:
-        tls_cert = os.path.join(tls_dir, my_cert)
-    if os.path.isabs(my_priv_key):
-        tls_priv_key = my_priv_key
-    else:
-        tls_priv_key = os.path.join(tls_dir, my_priv_key)
-
-    tls_cert_info = (tls_cert, tls_priv_key)
-
-
-def getData(registrar_ip, registrar_port, agent_id):
+    :returns: JSON structure containing the agent data
+    """
     # make absolutely sure you don't ask for data that contains AIK keys unauthenticated
-    if not tls_enabled:
+    if not tls_context:
         raise Exception("It is unsafe to use this interface to query AIKs without server-authenticated TLS.")
 
     response = None
     try:
-        client = RequestsClient(f"{registrar_ip}:{registrar_port}", tls_enabled, ignore_hostname=True)
-        response = client.get(f"/v{api_version}/agents/{agent_id}", cert=tls_cert_info, verify=ca_cert)
+        client = RequestsClient(f"{registrar_ip}:{registrar_port}", True, tls_context=tls_context)
+        response = client.get(f"/v{api_version}/agents/{agent_id}")
         response_body = response.json()
 
         if response.status_code == 404:
@@ -120,7 +90,20 @@ def getData(registrar_ip, registrar_port, agent_id):
             logger.critical("Error: did not receive port from Registrar Server.")
             return None
 
-        return response_body["results"]
+        r = response_body["results"]
+        res: RegistrarData = {
+            "aik_tpm": r["aik_tpm"],
+            "regcount": r["regcount"],
+            "ek_tpm": r["ek_tpm"],
+            "ip": r["ip"],
+            "port": r["port"],
+            "mtls_cert": r.get("mtls_cert"),
+            "ekcert": r.get("ekcert"),
+        }
+        if "provider_keys" in r:
+            res["provider_keys"] = r["provider_keys"]
+
+        return res
 
     except AttributeError as e:
         if response and response.status_code == 503:
@@ -135,9 +118,25 @@ def getData(registrar_ip, registrar_port, agent_id):
 
 
 def doRegisterAgent(
-    registrar_ip, registrar_port, agent_id, ek_tpm, ekcert, aik_tpm, mtls_cert=None, contact_ip=None, contact_port=None
-):
-    data = {
+    registrar_ip: str,
+    registrar_port: str,
+    agent_id: str,
+    ek_tpm: bytes,
+    ekcert: Optional[Union[bytes, str]],
+    aik_tpm: bytes,
+    mtls_cert: Optional[bytes] = None,
+    contact_ip: Optional[str] = None,
+    contact_port: Optional[str] = None,
+) -> Optional[str]:
+    """
+    Register the agent with the registrar
+
+    This is called by the agent code
+
+    :returns: base64 encoded blob containing the aik_tpm name and a challenge. Is encrypted with ek_tpm.
+    """
+
+    data: Dict[str, Any] = {
         "ekcert": ekcert,
         "aik_tpm": aik_tpm,
     }
@@ -147,6 +146,7 @@ def doRegisterAgent(
     if mtls_cert is not None:
         data["mtls_cert"] = mtls_cert
     else:
+        data["mtls_cert"] = "disabled"
         logger.error("Most actions require the agent to have mTLS enabled, but no cert was provided!")
     if contact_ip is not None:
         data["ip"] = contact_ip
@@ -155,10 +155,12 @@ def doRegisterAgent(
 
     response = None
     try:
-        client = RequestsClient(f"{registrar_ip}:{registrar_port}", tls_enabled, ignore_hostname=True)
-        response = client.post(
-            f"/v{api_version}/agents/{agent_id}", cert=tls_cert_info, data=json.dumps(data), verify=ca_cert
-        )
+        # The agent accesses the registrar without mTLS, meaning without client
+        # certificate
+        # TODO the registrar could be accessed using TLS, but without client
+        # certificate verification. Currently it is accessed without TLS at all
+        client = RequestsClient(f"{registrar_ip}:{registrar_port}", False)
+        response = client.post(f"/v{api_version}/agents/{agent_id}", data=json.dumps(data))
         response_body = response.json()
 
         if response.status_code != 200:
@@ -176,7 +178,7 @@ def doRegisterAgent(
             logger.critical("Error: did not receive blob from Registrar Server: %s", response.status_code)
             return None
 
-        return response_body["results"]["blob"]
+        return str(response_body["results"]["blob"])
     except Exception as e:
         if response and response.status_code == 503:
             logger.error("Agent cannot establish connection to registrar at %s:%s", registrar_ip, registrar_port)
@@ -187,13 +189,28 @@ def doRegisterAgent(
     return None
 
 
-def doActivateAgent(registrar_ip, registrar_port, agent_id, key):
+def doActivateAgent(registrar_ip: str, registrar_port: str, agent_id: str, key: bytes) -> bool:
+    """
+    Activate the agent with the registrar
+
+    Contact the registrar to inform the agent has the derived key
+
+    This is called by the agent code
+
+    :returns:
+    """
     data = {
         "auth_tag": crypto.do_hmac(key, agent_id),
     }
-    client = RequestsClient(f"{registrar_ip}:{registrar_port}", tls_enabled, ignore_hostname=True)
+
+    # The agent accesses the registrar without mTLS, meaning without client
+    # certificate
+    # TODO the registrar could be accessed using TLS, but without client
+    # certificate verification. Currently it is accessed without TLS at all
+    client = RequestsClient(f"{registrar_ip}:{registrar_port}", False)
     response = client.put(
-        f"/v{api_version}/agents/{agent_id}/activate", cert=tls_cert_info, data=json.dumps(data), verify=ca_cert
+        f"/v{api_version}/agents/{agent_id}/activate",
+        data=json.dumps(data),
     )
     response_body = response.json()
 
@@ -206,10 +223,20 @@ def doActivateAgent(registrar_ip, registrar_port, agent_id, key):
     return False
 
 
-def doRegistrarDelete(registrar_ip, registrar_port, agent_id):
-    client = RequestsClient(f"{registrar_ip}:{registrar_port}", tls_enabled, ignore_hostname=True)
-    response = client.delete(f"/v{api_version}/agents/{agent_id}", cert=tls_cert_info, verify=ca_cert)
-    response_body = response.json()
+def doRegistrarDelete(
+    registrar_ip: str, registrar_port: str, agent_id: str, tls_context: Optional[ssl.SSLContext]
+) -> Dict[str, Any]:
+    """
+    Delete the given agent from the registrar.
+
+    This is called by the tenant code
+
+    :returns: The request response body
+    """
+
+    client = RequestsClient(f"{registrar_ip}:{registrar_port}", True, tls_context=tls_context)
+    response = client.delete(f"/v{api_version}/agents/{agent_id}")
+    response_body: Dict[str, Any] = response.json()
 
     if response.status_code == 200:
         logger.debug("Registrar deleted.")
@@ -220,10 +247,19 @@ def doRegistrarDelete(registrar_ip, registrar_port, agent_id):
     return response_body
 
 
-def doRegistrarList(registrar_ip, registrar_port):
-    client = RequestsClient(f"{registrar_ip}:{registrar_port}", tls_enabled, ignore_hostname=True)
-    response = client.get(f"/v{api_version}/agents/", cert=tls_cert_info, verify=ca_cert)
-    response_body = response.json()
+def doRegistrarList(
+    registrar_ip: str, registrar_port: str, tls_context: Optional[ssl.SSLContext]
+) -> Optional[Dict[str, Any]]:
+    """
+    Get the list of registered agents from the registrar.
+
+    This is called by the tenant code
+
+    :returns: The request response body
+    """
+    client = RequestsClient(f"{registrar_ip}:{registrar_port}", True, tls_context=tls_context)
+    response = client.get(f"/v{api_version}/agents/")
+    response_body: Dict[str, Any] = response.json()
 
     if response.status_code != 200:
         logger.warning("Registrar returned: %s Unexpected response from registrar.", response.status_code)
