@@ -1,8 +1,3 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 import base64
 import http.server
 import ipaddress
@@ -12,15 +7,16 @@ import sys
 import threading
 from http.server import BaseHTTPRequestHandler, HTTPServer
 from socketserver import ThreadingMixIn
+from typing import Any, Dict, Tuple, cast
 
-from cryptography.hazmat.backends import default_backend
-from cryptography.x509 import load_der_x509_certificate
+from cryptography.hazmat.primitives.asymmetric import ec, rsa
 from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.orm.exc import NoResultFound
 
 from keylime import api_version as keylime_api_version
-from keylime import crypto, json, keylime_logging, web_util
+from keylime import cert_utils, config, crypto, json, keylime_logging, web_util
 from keylime.common import validators
+from keylime.da import record
 from keylime.db.keylime_db import DBEngineManager, SessionManager
 from keylime.db.registrar_db import RegistrarMain
 from keylime.tpm import tpm2_objects
@@ -35,17 +31,25 @@ except SQLAlchemyError as err:
     logger.error("Error creating SQL engine: %s", err)
     sys.exit(1)
 
+try:
+    rmc = record.get_record_mgt_class(config.get("registrar", "durable_attestation_import", fallback=""))
+    if rmc:
+        rmc = rmc("registrar")
+except record.RecordManagementException as rme:
+    logger.error("Error initializing Durable Attestation: %s", rme)
+    sys.exit(1)
+
 
 class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
-    def do_HEAD(self):
+    def do_HEAD(self) -> None:
         """HEAD not supported"""
         web_util.echo_json_response(self, 405, "HEAD not supported")
 
-    def do_PATCH(self):
+    def do_PATCH(self) -> None:
         """PATCH not supported"""
         web_util.echo_json_response(self, 405, "PATCH not supported")
 
-    def do_GET(self):
+    def do_GET(self) -> None:
         """This method handles the GET requests to retrieve status on agents from the Registrar Server.
 
         Currently, only agents resources are available for GETing, i.e. /agents. All other GET uri's
@@ -58,7 +62,7 @@ class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
             web_util.echo_json_response(self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
+        if not web_util.validate_api_version(self, cast(str, rest_params["api_version"]), logger):
             return
 
         if "agents" not in rest_params:
@@ -76,10 +80,12 @@ class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
                 logger.error("GET received an invalid agent ID: %s", agent_id)
                 return
 
+            agent = None
             try:
                 agent = session.query(RegistrarMain).filter_by(agent_id=agent_id).first()
             except SQLAlchemyError as e:
-                logger.error("SQLAlchemy Error: %s", e)
+                logger.error("SQLAlchemy Error for agent ID %s: %s", agent_id, e)
+                return
 
             if agent is None:
                 web_util.echo_json_response(self, 404, f"agent {agent_id} not found")
@@ -115,15 +121,15 @@ class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
 
         return
 
-    def do_POST(self):
+    def do_POST(self) -> None:
         """POST not supported"""
         web_util.echo_json_response(self, 405, "POST not supported via TLS interface")
 
-    def do_PUT(self):
+    def do_PUT(self) -> None:
         """PUT not supported"""
         web_util.echo_json_response(self, 405, "PUT not supported via TLS interface")
 
-    def do_DELETE(self):
+    def do_DELETE(self) -> None:
         """This method handles the DELETE requests to remove agents from the Registrar Server.
 
         Currently, only agents resources are available for DELETEing, i.e. /agents. All other DELETE uri's will return errors.
@@ -135,7 +141,7 @@ class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
             web_util.echo_json_response(self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
+        if not web_util.validate_api_version(self, cast(str, rest_params["api_version"]), logger):
             return
 
         if "agents" not in rest_params:
@@ -169,20 +175,20 @@ class ProtectedHandler(BaseHTTPRequestHandler, SessionManager):
         web_util.echo_json_response(self, 404)
 
     # pylint: disable=W0622
-    def log_message(self, format, *args):
+    def log_message(self, format: str, *args: Any) -> None:
         return
 
 
 class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
-    def do_HEAD(self):
+    def do_HEAD(self) -> None:
         """HEAD not supported"""
         web_util.echo_json_response(self, 405, "HEAD not supported")
 
-    def do_PATCH(self):
+    def do_PATCH(self) -> None:
         """PATCH not supported"""
         web_util.echo_json_response(self, 405, "PATCH not supported")
 
-    def do_GET(self):
+    def do_GET(self) -> None:
         """This method handles the GET requests to the unprotected side of the Registrar Server
 
         Currently the only supported path is /versions which shows the supported API versions
@@ -204,7 +210,7 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
 
         web_util.echo_json_response(self, 200, "Success", version_info)
 
-    def do_POST(self):
+    def do_POST(self) -> None:
         """This method handles the POST requests to add agents to the Registrar Server.
 
         Currently, only agents resources are available for POSTing, i.e. /agents. All other POST uri's
@@ -217,7 +223,7 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
             web_util.echo_json_response(self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
+        if not web_util.validate_api_version(self, cast(str, rest_params["api_version"]), logger):
             return
 
         if "agents" not in rest_params:
@@ -266,15 +272,10 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
                 # Note, we don't validate the EKCert here, other than the implicit
                 #  "is it a valid x509 cert" check. So it's still untrusted.
                 # This will be validated by the tenant.
-                ek509 = load_der_x509_certificate(
-                    base64.b64decode(ekcert),
-                    backend=default_backend(),
-                )
-                ek_tpm = base64.b64encode(
-                    tpm2_objects.ek_low_tpm2b_public_from_pubkey(
-                        ek509.public_key(),
-                    )
-                ).decode()
+                cert = cert_utils.x509_der_cert(base64.b64decode(ekcert))
+                pubkey = cert.public_key()
+                assert isinstance(pubkey, (rsa.RSAPublicKey, ec.EllipticCurvePublicKey))
+                ek_tpm = base64.b64encode(tpm2_objects.ek_low_tpm2b_public_from_pubkey(pubkey)).decode()
 
             aik_attrs = tpm2_objects.get_tpm2b_public_object_attributes(
                 base64.b64decode(aik_tpm),
@@ -290,11 +291,17 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
                 return
 
             # try to encrypt the AIK
-            (blob, key) = initialize_tpm.encryptAIK(
+            aik_enc = initialize_tpm.encryptAIK(
                 agent_id,
                 base64.b64decode(ek_tpm),
                 base64.b64decode(aik_tpm),
             )
+            if aik_enc is not None:
+                (blob, key) = aik_enc
+            else:
+                logger.warning("Agent %s failed encrypting AIK", agent_id)
+                web_util.echo_json_response(self, 400, "Error: failed encrypting AK")
+                return
 
             # special behavior if we've registered this uuid before
             regcount = 1
@@ -307,7 +314,6 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
                 raise
 
             if agent is not None:
-
                 # keep track of how many ek-ekcerts have registered on this uuid
                 regcount = agent.regcount
                 if agent.ek_tpm != ek_tpm or agent.ekcert != ekcert:
@@ -348,11 +354,11 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
 
             # Check for mTLS cert
             mtls_cert = json_body.get("mtls_cert", None)
-            if mtls_cert is None:
+            if mtls_cert is None or mtls_cert == "disabled":
                 logger.warning("Agent %s did not send a mTLS certificate. Most operations will not work!", agent_id)
 
             # Add values to database
-            d = {}
+            d: Dict[str, Any] = {}
             d["agent_id"] = agent_id
             d["ek_tpm"] = ek_tpm
             d["aik_tpm"] = aik_tpm
@@ -373,6 +379,14 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
                 logger.error("SQLAlchemy Error: %s", e)
                 raise
 
+            if rmc:
+                try:
+                    rmc.record_create(d, None, None)
+
+                except Exception as e:
+                    logger.error("Durable Attestation Error: %s", e)
+                    raise
+
             response = {
                 "blob": blob,
             }
@@ -384,7 +398,7 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
             logger.warning("POST for %s returning 400 response. Error: %s", agent_id, e)
             logger.exception(e)
 
-    def do_PUT(self):
+    def do_PUT(self) -> None:
         """This method handles the PUT requests to add agents to the Registrar Server.
 
         Currently, only agents resources are available for PUTing, i.e. /agents. All other PUT uri's
@@ -396,7 +410,7 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
             web_util.echo_json_response(self, 405, "Not Implemented: Use /agents/ interface")
             return
 
-        if not web_util.validate_api_version(self, rest_params["api_version"], logger):
+        if not web_util.validate_api_version(self, cast(str, rest_params["api_version"]), logger):
             return
 
         if "agents" not in rest_params:
@@ -458,12 +472,12 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
             logger.exception(e)
             return
 
-    def do_DELETE(self):
+    def do_DELETE(self) -> None:
         """DELETE not supported"""
         web_util.echo_json_response(self, 405, "DELETE not supported")
 
     # pylint: disable=W0622
-    def log_message(self, format, *args):
+    def log_message(self, format: str, *args: Any) -> None:
         return
 
 
@@ -474,15 +488,15 @@ class UnprotectedHandler(BaseHTTPRequestHandler, SessionManager):
 class RegistrarServer(ThreadingMixIn, HTTPServer):
     """Handle requests in a separate thread."""
 
-    def __init__(self, server_address, RequestHandlerClass):
+    def __init__(self, server_address: Tuple[str, int], RequestHandlerClass: Any) -> None:
         """Constructor overridden to provide ability to read file"""
         http.server.HTTPServer.__init__(self, server_address, RequestHandlerClass)
 
-    def shutdown(self):
+    def shutdown(self) -> None:
         http.server.HTTPServer.shutdown(self)
 
 
-def start(host, tlsport, port):
+def start(host: str, tlsport: int, port: int) -> None:
     """Main method of the Registrar Server.  This method is encapsulated in a function for packaging to allow it to be
     called as a function by an external program."""
 
@@ -500,7 +514,7 @@ def start(host, tlsport, port):
 
     # Set up the protected registrar server
     protected_server = RegistrarServer((host, tlsport), ProtectedHandler)
-    context, _ = web_util.init_mtls(section="registrar", generatedir="reg_ca", logger=logger)
+    context = web_util.init_mtls("registrar", logger=logger)
     if context is not None:
         protected_server.socket = context.wrap_socket(protected_server.socket, server_side=True)
     thread_protected_server = threading.Thread(target=protected_server.serve_forever)
@@ -514,7 +528,7 @@ def start(host, tlsport, port):
     thread_protected_server.start()
     thread_unprotected_server.start()
 
-    def signal_handler(signum, frame):
+    def signal_handler(signum: int, frame: Any) -> None:
         del signum, frame
         logger.info("Shutting down Registrar Server...")
         protected_server.shutdown()
