@@ -1,29 +1,22 @@
-#!/usr/bin/env python
-
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
+import asyncio
 import os
 import time
+from typing import Dict
 
 from keylime import ca_util
 from keylime import config as common
 from keylime import keylime_logging, secure_mount, tornado_requests
 
-# read the config file
-config = common.get_config()
 logger = keylime_logging.init_logging("update_crl")
 
 
-def execute(json_revocation):
+def execute(json_revocation: Dict[str, str]) -> None:
     if json_revocation["type"] != "revocation":
         return
 
     secdir = secure_mount.mount()
 
-    cert_path = config.get("cloud_agent", "revocation_cert")
+    cert_path = common.get("agent", "revocation_cert")
     if cert_path == "default":
         cert_path = os.path.join(secdir, "unzipped", "RevocationNotifier-cert.crt")
     else:
@@ -42,7 +35,9 @@ def execute(json_revocation):
     updated = False
     for _ in range(10):
         logger.debug("Getting updated CRL from %s", dist_path)
-        response = tornado_requests.request("GET", dist_path, None, None, None)
+        response = asyncio.get_event_loop().run_until_complete(
+            tornado_requests.request("GET", dist_path, None, None, None)  # type: ignore
+        )
         if response.status_code != 200:
             logger.warning("Unable to get updated CRL from %s.  Code %d", dist_path, response.status_code)
             time.sleep(1)
@@ -55,6 +50,7 @@ def execute(json_revocation):
         # write out the updated CRL
         logger.debug("Updating CRL in %s/unzipped/cacrl.der", secdir)
         with open(os.path.join(secdir, "unzipped", "cacrl.der"), "wb") as f:
+            assert not isinstance(response.body, str)
             f.write(response.body)
         ca_util.convert_crl_to_pem(
             os.path.join(secdir, "unzipped", "cacrl.der"), os.path.join(secdir, "unzipped", "cacrl.pem")
