@@ -1,9 +1,3 @@
-#!/usr/bin/python3
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-"""
-
 # important notice: this file was temporarly added to keylime in order to cover the gap
 # left by some additiomal features still not available on Intel's tpm2-tools. There is
 # an ongoing effort to add the aforementioned features to their toolkit and when this
@@ -52,14 +46,14 @@ yaml.add_representer(hexint, representer)
 efivarlib_functions = CDLL(config.LIBEFIVAR)
 
 
-def getDevicePath(b):
-    ret = efivarlib_functions.efidp_format_device_path(0, 0, b, len(b))
+def getDevicePath(b, l):
+    ret = efivarlib_functions.efidp_format_device_path(0, 0, b, l)
     if ret < 0:
         raise Exception(f"getDevicePath: efidp_format_device_path({b}) returned {ret}")
 
     s = create_string_buffer(ret + 1)
 
-    ret = efivarlib_functions.efidp_format_device_path(s, ret + 1, b, len(b))
+    ret = efivarlib_functions.efidp_format_device_path(s, ret + 1, b, l)
     if ret < 0:
         raise Exception(f"getDevicePath: efidp_format_device_path({b}) returned {ret}")
 
@@ -71,6 +65,7 @@ def getGUID(b):
     ret = efivarlib_functions.efi_guid_to_str(b, byref(s))
     if ret < 0:
         raise Exception(f"getGUID: efi_guid_to_str({b}) returned {ret}")
+    assert isinstance(s.value, bytes)
     return s.value.decode("utf-8")  # pylint: disable=E1101
 
 
@@ -180,7 +175,7 @@ def getVar(event, b):
                     c = w.decode("utf-16", errors="ignore")
                     description += c
                 r["Description"] = description
-                devicePath = getDevicePath(b[i:])
+                devicePath = getDevicePath(b[i:], len(b[i:]))
                 r["DevicePath"] = devicePath
                 return r
     return None
@@ -190,10 +185,11 @@ def enrich_device_path(d: dict) -> None:
     if isinstance(d.get("DevicePath"), str):
         try:
             b = bytes.fromhex(d["DevicePath"])
+            l = int(d["LengthOfDevicePath"])
         except Exception:
             return
         try:
-            p = getDevicePath(b)
+            p = getDevicePath(b, l)
         # Deal with garbage devicePath
         except Exception:
             return
